from EasyNN.examples.cifar10.structure import model
#from EasyNN.examples.cifar10.trained import model
from EasyNN.examples.cifar10.data import dataset

# Set your models data
model.training.data = dataset
